from collections import defaultdict
import lib2to3.fixer_util
import lib2to3.pgen2
from lib2to3.pygram import python_symbols as syms
from lib2to3.pytree import Node

from . import BaseFix
from my2to3.trace import create_table, register_tracing_function
from my2to3.util import parse_type


insert_trace = create_table("round_trace", "filename", "lineno", "id", "number", "ndigits")
insert_modified = create_table("round_modified", "filename", "lineno", "id")


@register_tracing_function
def round_traced(filename, lineno, id_, number, ndigits=0):
  insert_trace(
    filename,
    lineno,
    id_,
    number,
    ndigits
  )
  return round(number, ndigits)


class FixRoundTrace(BaseFix):
  """Rewrites round(a[, b]) into round_traced(id, a[, b])
  """
  # Inspired by https://github.com/python/cpython/blob/e42b705188271da108de42b55d9344642170aa2b/Lib/lib2to3/fixes/fix_xrange.py#L14
  PATTERN = """
            power<'round' trailer< '(' args=any ')' >
            rest=any* >
            """

  def transform(self, node, results):
    args = results['args']
    if args.type == lib2to3.pgen2.token.NUMBER:
      # e.g. round(1.23)
      children = [
        args.clone()
      ]
    else:
      # e.g. round(1,23, 4)
      children = [leaf.clone() for leaf in args.children]

    node.replace(self.traced_call("division_traced", insert_modified, node, children))
